/*
 * xplay --- X sound player.
 *
 * This program is copyright 1994 by Robert S. Thau.  It is offered as
 * is, and no warranty is offered of suitability or fitness for any
 * particular purpose; use at your own risk.  Contents may have settled
 * during shipment.  The program may be redistributed for noncommercial
 * purposes only, so long as all files contain a prominent notice of when
 * they were last changed, and who changed them.
 */

/*
 * The "format" structure encapsulates everything the program knows
 * about a particular sound sample --- details of the format, and where
 * the actual data starts and ends in the file.
 *
 * The program handles data formats which the sound chip can't handle
 * by converting it piecemeal into relatively small buffers.
 *
 * So, we have one of these structures for the raw input, and one for the
 * input after conversion.  See play.c.
 */
     
enum audio_encoding { CODE_ULAW, CODE_ALAW,
			CODE_LINEAR, CODE_LINEAR_UNSIGNED };
     
struct format
{
  int data_offset;
  int num_samples;
  int file_size;		/* Actually offset of last sample;
				 * for AIFF and WAVE, this may not be actual
				 * EOF due to trailing ID chunks.
				 */

  enum audio_encoding coding;
  int nbits;
  int channels;
  int rate;
  int little_endian;
};

/* Initialize format structure from a file header.  
 * HDR_LENGTH is the length of the longest possible header ---
 * play.c is supposed to read in at least this much data before
 * calling grok_audio_header, and the format-specific header parsing
 * routines should not count on being able to see more.
 *
 * The char* return is for an error message, and is NULL on success.
 */

#define HDR_LENGTH 4096		/* Max hdr length, for ANY audio format */
char *grok_audio_header (struct format *, char *);

/* General accounting --- going from time to file offset (user has
 * positioned the slider), and file offset to number of samples
 * (for going the other way...).
 */

int samples_from_size (struct format *fmt, int file_size);
int offset_for_sample (struct format *fmt, int offset);     
