/*
 * xplay --- X sound player.
 *
 * This program is copyright 1994 by Robert S. Thau.  It is offered as
 * is, and no warranty is offered of suitability or fitness for any
 * particular purpose; use at your own risk.  Contents may have settled
 * during shipment.  The program may be redistributed for noncommercial
 * purposes only, so long as all files contain a prominent notice of when
 * they were last changed, and who changed them.
 */

/*
 * Format conversion routines.
 *
 * grok_for_conversion takes as arguments the
 * "format" structure describing the format of the incoming sound sample,
 * and initializes another one describing a format which the hardware
 * actually understands, which is easy to convert to.  The third argument
 * describes device specifications; right now, its format is general
 * enough to specify the Sun sound chips and no more.  See play.c for
 * details.
 *
 * grok_for_conversion returns a cookie which is to be passed to
 * convert_samples to enable the actual conversion.
 *
 * check_for_conversion checks to be sure that the incoming sample
 * format is something we can handle, returning NULL or an error message.
 *
 * convert_samples does the conversion itself.  Note that since we don't
 * necessarily know how many destination samples a set of source bytes
 * will expand into (not with compressed formats, we don't --- never mind
 * that we don't handle that yet!), we take length arguments for both
 * source and dest.  Also takes handles to dest & src so they may be
 * advanced by the # bytes produced & consumed, respectively.
 */
     
char *check_for_conversion (struct format *src);
void *grok_for_conversion(struct format *dst, struct format *src, int *device);
void convert_samples (char **dest, char **src, void *cookie,
		      int dst_nbytes, int src_nbytes);
     


