#include <stdio.h>
#include <malloc.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>

#include "file.h"

/*
 * xplay --- X sound player.
 *
 * This program is copyright 1994 by Robert S. Thau.  It is offered as
 * is, and no warranty is offered of suitability or fitness for any
 * particular purpose; use at your own risk.  Contents may have settled
 * during shipment.  The program may be redistributed for noncommercial
 * purposes only, so long as all files contain a prominent notice of when
 * they were last changed, and who changed them.
 */

#define CHUNK_SIZE (1<<19)

struct xfile_chunk *alloc_chunk(offset)
{
  struct xfile_chunk *new_chunk =
    (struct xfile_chunk *)malloc(sizeof(struct xfile_chunk));

  if (new_chunk == NULL) return NULL;

  new_chunk->data = malloc(CHUNK_SIZE);

  if (new_chunk->data == NULL)
  {
    free ((char *)new_chunk);
    return NULL;
  }

  new_chunk->offset_in_file = offset;
  new_chunk->next = NULL;
  new_chunk->data_end = new_chunk->data + CHUNK_SIZE;
  new_chunk->good_data_end = new_chunk->data;

  return new_chunk;
}

void read_whats_available (struct xfile *xf)
{
  struct xfile_chunk *x = xf->last;
  
  long bytes_left = x->data_end - x->good_data_end;
  int retval;
  
  if (xf->eof) return;
  
  retval = read (xf->fd, x->good_data_end, bytes_left);

  if (retval < 0) return;	/* EAGAIN --- no data available this time. */
    
  if (retval == 0)
  {
    xf->eof = 1;
    return;
  }

  bytes_left -= retval;
  x->good_data_end += retval;
  xf->total_bytes_read += retval;
  
  if ((bytes_left == 0)
      && (x->next = alloc_chunk(x->offset_in_file + (x->data_end - x->data))))
  {
    xf->last = x->next;
    read_whats_available (xf);
  }
}

/* Routines for handling I/O to and from files,
 * in particular, the interactive streams.
 */

static char *sperror (char *foo, int old_errno)
{
  extern char *sys_errlist[];
  
  int length = strlen(foo) + strlen (sys_errlist [old_errno]) + 10;
  char *buf = malloc (length);

  if (buf == NULL)
  {
    errno = old_errno;
    perror (foo);
    fprintf (stderr, "And out of memory besides!\n");
    exit (1);
  }

  strcpy (buf, foo);
  strcat (buf, ": ");
  strcat (buf, sys_errlist [old_errno]);

  return buf;
}

char *open_xfile (struct xfile *x, char *filename)
{
  int file;
  char *errmsg;
  
  if (!strcmp (filename, "-"))
  {
    file = 0;
  }
  else if ((file = open (filename, O_RDONLY)) < 0)
  {
    return sperror (filename, errno);
  }
  
  if ((x->first = alloc_chunk(0)) == NULL)
  {
    return "Couldn't malloc!";
  }

  x->fd = file;
  x->last = x->first;
  fcntl (file, F_SETFL, O_NONBLOCK);
  
  return NULL;
}

char *
save_xfile (struct xfile *x, char *filename)
{
  int fd = open (filename, O_CREAT|O_WRONLY|O_TRUNC, 0666);
  struct xfile_chunk *chunkp = x->first;
  int delta;

  if (fd < 0) return sperror (filename, errno);
 
  for (chunkp = x->first; chunkp != NULL; chunkp = chunkp->next)
  {
    char *buf_ptr = chunkp->data;
    int size = chunkp->good_data_end - chunkp->data;

    while (size > 0 && (delta = write (fd, buf_ptr, size)) > 0)
    {
      buf_ptr += delta;
      size -= delta;
    }

    if (delta < 0) break;
  }

  if (delta < 0)
  {
    int old = errno;
    close (fd);
    return sperror(filename, old);
  }

  close (fd);
  return NULL;
}

