#include <stdio.h>
#include "audio.h"

/*
 * xplay --- X sound player.
 *
 * This program is copyright 1994 by Robert S. Thau.  It is offered as
 * is, and no warranty is offered of suitability or fitness for any
 * particular purpose; use at your own risk.  Contents may have settled
 * during shipment.  The program may be redistributed for noncommercial
 * purposes only, so long as all files contain a prominent notice of when
 * they were last changed, and who changed them.
 */

extern char *grok_aiff (struct format *, char *);
extern char *grok_au (struct format *, char *);
extern char *grok_wave (struct format *, char *);

char *grok_audio_header (struct format *samp, char *header)
{
  char *errmsg = NULL;

  /* Figure out what the file looks like.  If it doesn't look like
   * anything much, assume raw ulaw samples at 8 kHz and hope for the best.
   */
  
  if (!memcmp (header, "FORM", 4)) errmsg = grok_aiff (samp, header);
  else if (!memcmp (header, ".snd", 4)) errmsg = grok_au (samp, header);
  else if (!memcmp (header, "RIFF", 4)) errmsg = grok_wave (samp, header);
  else
  {
    samp->data_offset = 0;
    samp->num_samples = -1;	/* unknown... */
    samp->file_size = -1;	/* unknown... */
    samp->coding = CODE_ULAW;
    samp->nbits = 8;
    samp->channels = 1;
    samp->rate = 8000;
  }

  return errmsg;
}

/* Another routine at the awkward interface between knowing about
 * files and knowing about audio... if a sound file doesn't size
 * information from its header, we need to be able to guess the
 * number of samples in the file from the number of bytes we've
 * read, in order to present playback position as a fraction of *that*.
 *
 * This code works with all currently supported formats...
 */

int samples_from_size (struct format *fmt, int file_size)
{
  file_size -= fmt->data_offset;
  file_size /= (fmt->nbits / 8);
  file_size /= fmt->channels;

  return file_size;
}

/* Another piece of bookkeeping --- file offset to get to a particular
 * position in tenth-of-second units.
 */

int offset_for_sample (struct format *fmt, int offset)
{
  int revised_offset = (offset * fmt->rate) / 10;

  int src_bytes_per_sample = fmt->channels * fmt->nbits / 8;
  
  return fmt->data_offset + revised_offset * src_bytes_per_sample;
}

