/*
 * xplay --- X sound player.
 *
 * This program is copyright 1994 by Robert S. Thau.  It is offered as
 * is, and no warranty is offered of suitability or fitness for any
 * particular purpose; use at your own risk.  Contents may have settled
 * during shipment.  The program may be redistributed for noncommercial
 * purposes only, so long as all files contain a prominent notice of when
 * they were last changed, and who changed them.
 */



/*
 * Audio player --- the following few functions constitute the gui's
 * interface to the code which actually runs the player.  Consider this
 * the gui "anti-header".  All of these are implemented in play.c.
 *
 * Those functions returning (char *) return NULL on success,
 * or an error message
 */

/*
 * Setting up --- audio_file_init sets up the play code to process
 * a particular file, or `-'.
 */

char *audio_file_init (char *filename);
int audio_good_port (char *);	/* Is this port name valid?
				 * (Output port == name for speaker;
				 * e.g., speakerbox, headphone, etc.)
				 */

/*****************************************************************
 *
 * Controlling playback.  The offset for audio_start is in "common time
 * units," of tenths of a second --- the same units returned by a lot
 * of the status inquiries below.
 *
 * play.c clips this offset to audio_last_posn_read(), if the user is
 * trying to go beyond.
 */
     
void audio_start (int offset, char *port);
void audio_abort (void);

int audio_is_over();		/* True when playback has run off
				 * the end.  Don't try waiting until
				 * audio_posn() is equal to audio_size();
				 * device driver weirdness may prevent that.
				 */
     
/*****************************************************************
 *
 * Reading in the file --- we count on gui.c to periodically sample
 * audio_fd(), and to call audio_read_some_more() if data is available,
 * until audio_is_all_read().  
 *
 * XtAppAddInput makes this easy, but there are other ways to get the
 * same effect.
 */
     
int audio_is_all_read();	/* Never blocks. */
void audio_read_some_more();
int audio_fd();		
     
/*****************************************************************
 *
 * Status inquiries.  Note that the value of audio_size can *change*
 * over time; the file header doesn't always give us reliable size
 * information, so we can't necessarily be sure unless audio_is_all_read().
 */
     
int audio_size();		/* In tenths of a second */
int audio_size_is_a_guess();	/* True if it may still change; never
				 * true if audio_is_all_read().
				 */
     
int audio_posn();		/* In tenths, for the slider. */

int audio_last_posn_read();     /* So we don't let the user fast-forward
				 * past if we can help it...
				 */
     
/*****************************************************************
 *
 * Controlling volume.  Self-explanatory.
 */

int audio_min_volume();
int audio_max_volume();
int audio_current_volume();
void audio_set_volume(int);     
     
/*****************************************************************
 *
 * Miscellaneous.
 */
     
char *audio_save (char *filename);

